/*

Copyright 2011 Giuseppe Cosentino Borgese <gicos.2@gmail.com>

This file is part of gSudoku.

gSudoku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

gSudoku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with gSudoku.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include "SudokuTable.h"

SudokuTable::SudokuTable(){
	resetTable();
}

bool SudokuTable::loadSudoku(string sSudoku){
	if(sSudoku.length() == 81){
		int x = 0, y = 0;
		for(int i = 0; i < 81; i++){
			x = i / 9;
			y = i % 9;
			sTable[x][y] = SudokuCell();
			int toIns = sSudoku[i] % 48;
			if (toIns > 0 && toIns < 10){
				sTable[x][y].setCellValue((int) sSudoku[i] % 48);
				sTable[x][y].setFixed(true);
			}
		}
		return true;
	}
	return false;
}

bool SudokuTable::solveTable(int x, int y){
	//Siamo usciti fuori dalla matrice? Abbiamo finito
	if(x > 8 || y > 8){
		return true;
	}
	//Calcoliamo le coordinate della cella non fissata successiva
	int nX = x, nY = y;
	do{
		nX = nX + ((nY + 1) / 9);
		nY = (nY + 1) % 9;
	}while (sTable[nX][nY].isFixed());

	//Non è una cella fissata? Calcoliamone la soluzione
	if(!sTable[x][y].isFixed()){
		//Prendiamo tutti i valori inseribili
		list<int> pValues = validValues(x, y);
		//Non ce ne sono? Resettiamo la cella e torniamo indietro di un passo
		if(pValues.empty()){
			sTable[x][y] = SudokuCell();
			return false;
		}
		//Prendiamo un valore a caso tra quelli inseribili
		int rNum = rand() % pValues.size();
		list<int>::iterator iR = pValues.begin();
		for(int i = 0; i < rNum; i++){
			iR++;
		}
		//Inseriamone il valore nella cella
		sTable[x][y].setCellValue(*iR);
		//Proviamo a risolvere la cella successiva
		bool trySolve = solveTable(nX, nY);
		//Non è possibile?
		if(!trySolve){
			//Resettiamo la cella successiva
			sTable[nX][nY] = SudokuCell();
			//Aggiungiamo il valore scelto prima a quelli non possibili
			sTable[x][y].addInvalidValue(*iR);
			//Proviamo con un altro numero
			return solveTable(x, y);
		}else{
			return trySolve;
		}
	} else { //Altrimenti passiamo alla prossima cella non fissata da risolvere
		return solveTable(nX, nY);
	}
}

list<int> SudokuTable::validValues(int x, int y){
	int secX = x - (x % 3), secY = y - (y % 3);
	list<int> pValues, cellInvalidValues = sTable[x][y].getInvalidValues();
	for(int i = 1; i < 10; i++){
		pValues.push_back(i);
	}

	for(int i = 0; i < 9; i++){
		pValues.remove(sTable[i][y].getCellValue());
		pValues.remove(sTable[x][i].getCellValue());
	}
	for(int i = secX; i < (secX + 3); i++){
		for(int j = secY; j < (secY + 3); j++){
			pValues.remove(sTable[i][j].getCellValue());
		}
	}
	if(!cellInvalidValues.empty()){
		for(list<int>::iterator iV = cellInvalidValues.begin(); iV != cellInvalidValues.end(); iV++){
			pValues.remove(*iV);
		}
	}
	return pValues;
}

void SudokuTable::resetTable(){
	for(int i = 0; i < 9; i++){
		for(int j = 0; j < 9; j++){
			sTable[i][j] = SudokuCell();
		}
	}
}

void SudokuTable::setCellValue(int x, int y, int val){
	sTable[x][y].setCellValue(val);
}

int SudokuTable::getCellValue(int x, int y){
	return sTable[x][y].getCellValue();
}

bool SudokuTable::isCellFixed(int x, int y){
	return sTable[x][y].isFixed();
}

