/*

Copyright 2011 Giuseppe Cosentino Borgese <gicos.2@gmail.com>

This file is part of gSudoku.

gSudoku is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

gSudoku is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with gSudoku.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include "GUISudoku.h"
#include "GUIError.h"

GUISudoku::GUISudoku(WINDOW *mW = NULL, int mY = 0, int mX = 0, int nLvl = 0):
	mainWin(mW), maxX(mX), maxY(mY), checkErrors(false), selX(-1), selY(-1)
{
	menuItems.push_back("+");
	menuItems.push_back("[");
	menuItems.push_back("  Risolvi cella");
	menuItems.push_back("  Risolvi settore");
	menuItems.push_back("  Risolvi Sudoku");
	menuItems.push_back("  Ricomincia");
	menuItems.push_back("  Esci");

	sMax = menuItems.size();
	wHei = (2 * sMax) + 1;
	wWid = 22;
	errPosX = maxX - 31;
	errPosY = wHei + 3;

	optionsWin = subwin(mainWin, wHei, wWid, 1, (maxX - wWid) - 4);
	sudokuWin = subwin(mainWin, 19, 39, 1, 3);
	hintsWin = subwin(mainWin, 3, 39, 20, 3);

	SudokuDB dbS;
	do{
		sdUnsolved.loadSudoku(dbS.readSudoku(nLvl));
		sdSolved = sdUser = sdUnsolved;
	}while(!sdSolved.solveTable(0, 0)); //Assicuriamoci sia risolvibile!
}

void GUISudoku::playSudoku(){
	clear();
	drawOptionsWin(-1);
	drawSudokuWin();
	refresh();
	useOptionsWin();
}

void GUISudoku::drawSudokuWin(){
	if(selX < 0 || selY < 0){
		for(int i = 1; i < 18; i++){
			if(i % 2){
				mvwaddstr(sudokuWin, i, 1, "   |   |   ||   |   |   ||   |   |   ");
			}else{
				if(i % 6){
					mvwaddstr(sudokuWin, i, 1, "---+---+---||---+---+---||---+---+---");
				}else{
					mvwaddstr(sudokuWin, i, 1, "===========][===========][===========");
				}
			}
		}
		box(sudokuWin, 0, 0);
		selX = selY = 0;
	}
	string toPrint;
	for(int i = 0; i < 9; i++){
		for(int j = 0; j < 9; j++){
			int cVal = sdUser.getCellValue(i, j);
			if(selX == i && selY == j){
				wcolor_set(sudokuWin, 2, NULL);
			}else if(((selX == i || selY == j) || (i/3 == selX/3 && j/3 == selY/3)) && showHints){
				if(sdUser.isCellFixed(i,j)){
					wcolor_set(sudokuWin, 8, NULL);
				}else if(cVal > 0 && checkErrors){
					if(cVal != sdSolved.getCellValue(i, j)){
						wcolor_set(sudokuWin, 5, NULL);
					}else{
						wcolor_set(sudokuWin, 9, NULL);
					}
				}else{
					wcolor_set(sudokuWin, 7, NULL);
				}
			}else if(sdUser.isCellFixed(i,j)){
				wcolor_set(sudokuWin, 3, NULL);
			}else if(cVal > 0 && checkErrors){
				int scVal = sdSolved.getCellValue(i, j);
				if(cVal != scVal){
					wcolor_set(sudokuWin, 5, NULL);
				}else if(cVal == scVal){
					wcolor_set(sudokuWin, 4, NULL);
				}
			}
			mvwaddstr(sudokuWin, (2*i) + 1, 1 + 4*j + j/3, "   ");
			if(cVal > 0){
				mvwaddch(sudokuWin, (2*i) + 1, 2 + 4*j + j/3, cVal + 48);
			}
			wcolor_set(sudokuWin, 1, NULL);
		}
	}
	wrefresh(sudokuWin);
	drawHintsWin();
}

void GUISudoku::drawOptionsWin(int sMenu){
	string toPrint, sToggleErrors;

	box(optionsWin, 0 , 0);
	for(int i = 0; i < (wHei - 2); i++){
		if(i % 2 == 0){
			int k = i / 2;
			if(k == sMenu){
				wcolor_set(optionsWin, 2, NULL);
			}
			mvwaddstr(optionsWin, (i + 1), 1, "                    ");
			switch(menuItems[k][0]){
			case '+':
				toPrint = "  Hints: ";
				toPrint.append((showHints) ? "Si" : "No");
				break;
			case '[':
				toPrint = "  Check errori: ";
				toPrint.append((checkErrors) ? "Si" : "No");
				break;
			default:
				toPrint = menuItems[k];
				break;
			}
		}else{
			toPrint = "--------------------";
		}
		mvwaddstr(optionsWin, (i + 1), 1, &toPrint[0]);
		wcolor_set(optionsWin, 1, NULL);
	}
	wrefresh(optionsWin);
}

void GUISudoku::drawHintsWin(){
	if(showHints){
		box(hintsWin, 0, 0);
		mvwaddstr(hintsWin, 1, 1, "   |   |   ||   |   |   ||   |   |   ");
		if((selX >= 0 && selY >= 0) && !sdUser.isCellFixed(selX, selY)){
			list<int> pVal = sdUser.validValues(selX, selY);
			for(list<int>::iterator iT = pVal.begin(); iT != pVal.end(); iT++){
				mvwaddch(hintsWin, 1, 2 + 4*(*iT - 1) + (*iT - 1)/3, *iT + 48);
			}
		}
	}else{
		mvwaddstr(hintsWin, 0, 0, "                                       ");
		mvwaddstr(hintsWin, 1, 0, "                                       ");
		mvwaddstr(hintsWin, 2, 0, "                                       ");
	}
	wrefresh(hintsWin);
}

void GUISudoku::useOptionsWin(){
	int pKey, sMenu = 0;

	while(true){
		drawOptionsWin(sMenu);
		pKey = getch();
		switch(pKey){
		case KEY_UP:
			sMenu = ((sMenu - 1) < 0) ? (sMax - 1) : (sMenu - 1);
			break;
		case KEY_DOWN:
			sMenu = (sMenu + 1) % sMax;
			break;
		case '\n':
			switch(sMenu){
			case 0:
				showHints = !showHints;
				drawSudokuWin();
				break;
			case 1:
				checkErrors = !checkErrors;
				drawSudokuWin();
				break;
			case 2:
			{ //Senza parentesi non compila (cross-initializzation error): perchè? A quanto pare è necessario "dare" uno scope.
				GUIError e(mainWin, errPosX, errPosY, false);
				if(e.showError("Risolvere la cella?")){
					sdUser.setCellValue(selX, selY, sdSolved.getCellValue(selX, selY));
				}
			}
				drawSudokuWin();
				break;
			case 3:
				{
					GUIError e(mainWin, errPosX, errPosY, false);
					if(e.showError("Risolvere il settore?")){
						int secX = (selX/3)*3, secY = (selY/3)*3;
						for(int i = secX; i < (secX + 3); i++){
							for(int j = secY; j < (secY + 3); j++){
								sdUser.setCellValue(i, j, sdSolved.getCellValue(i, j));
							}
						}
					}
				}
				drawSudokuWin();
				break;
			case 4:
			{
				GUIError e(mainWin, errPosX, errPosY, false);
				if(e.showError("Risolvere tutte le celle?")){
					sdUser = sdSolved;
				}
			}
				drawSudokuWin();
				break;
			case 5:
			{
				GUIError e(mainWin, errPosX, errPosY, false);
				if(e.showError("Cancellare il Sudoku?")){
					sdUser = sdUnsolved;
				}
			}
				drawSudokuWin();
				break;
			case 6:
			{
				GUIError e(mainWin, errPosX, errPosY, false);
				if(e.showError("Uscire veramente?")){
					return;
				}
			}
				break;
			}
			break;
		case '\t':
			drawOptionsWin(-1);
			useSudokuWin();
			break;
		}
	}
}

void GUISudoku::useSudokuWin(){
	int pKey;
	selX = selY = 0;
	while(true){
		drawSudokuWin();
		pKey = getch();
		switch(pKey){
		case KEY_UP:
			selX = ((selX - 1) < 0) ? 8 : (selX - 1);
			break;
		case KEY_DOWN:
			selX = (selX + 1) % 9;
			break;
		case KEY_LEFT:
			selY = ((selY - 1) < 0) ? 8 : (selY - 1);
			break;
		case KEY_RIGHT:
			selY = (selY + 1) % 9;
			break;
		case '\n':
			if(sdUser.isCellFixed(selX, selY)){
				GUIError e(mainWin, errPosX, errPosY, true);
				e.showError("Cella non modificabile");
			}else{
				int chSel;
				do{
					char cellNo = sdUser.getCellValue(selX, selY);
					string cToPrint = "   ";
					wcolor_set(sudokuWin, 6, NULL);
					mvwaddstr(sudokuWin, (2*selX) + 1, 1 + 4*selY + selY/3, cToPrint.c_str());
					if(cellNo > 0){
						mvwaddch(sudokuWin, (2*selX) + 1, 2 + 4*selY + selY/3, cellNo + 48);
					}
					wrefresh(sudokuWin);
					chSel = getch();
					if(chSel >= '0' && chSel <= '9'){
						sdUser.setCellValue(selX, selY, chSel - 48);
					}
				}while(chSel != '\n');
				wcolor_set(sudokuWin, 1, NULL);
			}
			break;
		case '\t':
			return;
			break;
		}
	}
}

GUISudoku::~GUISudoku(){
	delwin(optionsWin);
	delwin(sudokuWin);
	delwin(hintsWin);
	refresh();
}
